/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wGaussTetra4.h"
#include <stdexcept>
#include <cmath>

using namespace tbox;

GaussTetra4::GaussTetra4(int n) : Gauss()
{
    switch (n)
    {
    case 1:
    {
        p.resize(1);
        w.resize(1);
        ngp = 1;
        p[0] = Eigen::Vector3d(0.25, 0.25, 0.25);
        w[0] = 1. / 6.;
        break;
    }
    case 2:
    {
        p.resize(4);
        w.resize(4); // 4 pts !
        ngp = 4;
        double a = (5. - sqrt(5.)) / 20.;
        double b = (5. + 3. * sqrt(5.)) / 20.;
        p[0] = Eigen::Vector3d(a, a, a);
        w[0] = 1. / 4. / 6.;
        p[1] = Eigen::Vector3d(a, a, b);
        w[1] = 1. / 4. / 6.;
        p[2] = Eigen::Vector3d(a, b, a);
        w[2] = 1. / 4. / 6.;
        p[3] = Eigen::Vector3d(b, a, a);
        w[3] = 1. / 4. / 6.;
        break;
    }
    default:
        throw std::runtime_error("not implemented (GaussTetra4)!");
    }
}
