/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wGaussTri3.h"
#include <stdexcept>
#include <cmath>

using namespace tbox;

GaussTri3::GaussTri3(int n) : Gauss()
{
    switch (n)
    {
    case 1:
        p.resize(1);
        w.resize(1);
        ngp = 1;
        p[0] = Eigen::Vector3d(1. / 3, 1. / 3, 0.0);
        w[0] = 1. / 2.; // ! sum=1/2
        break;
    case 2:
        p.resize(3);
        w.resize(3); // 3 pts !
        ngp = 3;
        p[0] = Eigen::Vector3d(1. / 6, 1. / 6, 0.0);
        w[0] = 1. / 3. / 2.;
        p[1] = Eigen::Vector3d(2. / 3, 1. / 6, 0.0);
        w[1] = 1. / 3. / 2.;
        p[2] = Eigen::Vector3d(1. / 6, 2. / 3, 0.0);
        w[2] = 1. / 3. / 2.;
        break;
    case 3:
        p.resize(4);
        w.resize(4);
        ngp = 4;
        p[0] = Eigen::Vector3d(1. / 3, 1. / 3, 0.0);
        w[0] = -27. / 48. / 2.;
        p[1] = Eigen::Vector3d(1. / 5, 3. / 5, 0.0);
        w[1] = 25. / 48. / 2.;
        p[2] = Eigen::Vector3d(1. / 5, 1. / 5, 0.0);
        w[2] = 25. / 48. / 2.;
        p[3] = Eigen::Vector3d(3. / 5, 1. / 5, 0.0);
        w[3] = 25. / 48. / 2.;
        break;
    default:
        throw std::runtime_error("not implemented (GaussTri3)!");
    }
}
