/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLAZY_H
#define WLAZY_H

#include "tbox.h"

#include <mutex>
#include <atomic>

namespace tbox
{

/**
 * @brief TBB implementation of a "lazy initialization"
 *        (thread safe allocation of a shared pointer)
 */

template <typename T>
class Lazy
{
    std::atomic<T *> value;
    std::mutex mut;
    int np;

public:
    Lazy(int _np) : value(), np(_np) {}
    ~Lazy()
    {
        delete value;
        //std::cout << "lazy pointer deleted\n";
    }
    T &get()
    {
        if (!value)
        {
            std::unique_lock<std::mutex> lock(mut);
            if (!value)
                value = new T(np);
        }
        return *value;
    }
};

} // namespace tbox

#endif //WLAZY_H
